/* ---------------------------------------------------------------------- */
/*                                DEMO3.C                                 */
/*                                                                        */
/*      Resolucao de equacoes polinomiais pelo metodo de Muller (1)       */
/*                                                                        */
/*                    (C) 1993, Lenimar Nunes de Andrade                  */
/*                          CCENDM03 @ BRUFPB.BITNET                      */
/*                                                                        */
/*       Veja  o  arquivo  SOLVEQ.DOC  para maiores informacoes sobre as  */
/* as funcoes e estruturas usados.                                        */
/*                                                                        */
/*        Para compilar este programa, use o TCC do Turbo C 2.0 e digite  */
/*                      [C:\] TCC -G -Z demo3 solveq.lib                  */
/*        Para salvar em disco a solucao digite                           */
/*                      [C:\] DEMO3 > NomeArq.Ext                         */
/* ---------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <ctype.h>
#include <string.h>

#include "solveq.h"

main () {

  /* --- Variaveis globais --- */
  polinomio p, p_copia, p_aux;
  complexo  z, r, *raizes;
  int RaizesEncontradas = 0, iteracoes, i;
  char coeficiente[50];

  /* --- Aproximacao inicial 1+i da primeira raiz de p --- */
  z.a = 1; z.b = 1;

  /* --- Leitura do grau do polinomio p --- */
  fprintf(stderr, "\nGrau da equao? ");
  scanf("%d", &p.grau);

  /* --- Aloca memoria para os coeficientes do polinomio --- */
  p.coef = (complexo *) malloc((p.grau + 1)*sizeof(complexo));

  /* --- Leitura dos coeficientes do polinomio p --- */
  printf("\n");
  for (i = p.grau; i >=0; i--) {
    fprintf(stderr, "Coeficiente do termo de grau %d : ", i);
    scanf("%s", coeficiente);
    p.coef[i] = ConvComp(coeficiente);
  }

  /* --- Aloca memoria para o polinomio p_copia e define-o como uma
         copia de p --- */
  p_copia.grau = p.grau;
  p_copia.coef = (complexo *) malloc((p.grau + 1)*sizeof(complexo));
  memcpy(p_copia.coef, p.coef, (p.grau + 1)*sizeof(complexo));

  /* --- Aloca memoria para o polinomio auxiliar p_aux --- */
  p_aux.grau = p.grau;
  p_aux.coef = (complexo *) malloc((p.grau + 1)*sizeof(complexo));

  /* --- Aloca memoria para as raizes do polinomio p --- */
  raizes = (complexo *) malloc(p.grau*sizeof(complexo));

  fprintf(stderr, "\nResolvendo...");

  /* --- Calculo das raizes de p --- */

  /*
   *  Raizes nulas devem ser consideradas separadamente
   */

  while (p.grau && Muller(&iteracoes, p, &z, 1E-6, 200, 0.5))
  {
    if (!Muller(&iteracoes, p_copia, &z, 1E-11, 100, 0.0001)) break;

    /* --- Guarda a raiz encontrada z em raizes e incrementa a quantidade de
           raizes encontradas --- */
    raizes[RaizesEncontradas] = z;
    RaizesEncontradas++;

    /* --- Divide p(x) por x - z. O quociente e' guardado em p_aux(x) --- */
    DividePol(p, z, &p_aux, &r);

    /* --- Copia p_aux em p --- */
    p.grau = p_aux.grau;
    memcpy(p.coef, p_aux.coef, (p_aux.grau + 1)*sizeof(complexo));
  }

  /* --- Mostra na tela os resultados encontrados --- */
  clrscr();
  puts("RESOLUCAO DE EQUACOES POLINOMIAIS USANDO O METODO DE MULLER\n");
  printf("Grau = %d\n", p_copia.grau);
  puts("\nCoeficientes:\n");

  /* --- Mostra os coeficientes de p_copia em um campo de largura 80. Se o
         valor for fracionario, entao ele sera' mostrado com 6 decimais --- */
  MostraPolinomio(p_copia, 80, 6);

  puts("\n\nRaizes:");
  printf("\n   #        parte real      parte imaginaria   ");
  printf("\n---------------------------------------------");
  for (i = 0; i < RaizesEncontradas; i++)
    printf("\n  %2d    %16.10Lf  %16.10Lf", i+1, raizes[i].a, raizes[i].b);
  printf("\n---------------------------------------------\n");

  /* --- Libera a memoria ocupada por raizes, p, p_copia e p_aux --- */
  free(raizes);
  free(p.coef);
  free(p_copia.coef);
  free(p_aux.coef);
}

/*** FIM DE "DEMO3.C" ***/
